@@ -1,8 +1,8 @@
 # $File: //member/autrijus/PAR/PAR.pm $ $Author: autrijus $
-# $Revision: #5 $ $Change: 1521 $ $DateTime: 2002/10/19 02:30:49 $
+# $Revision: #6 $ $Change: 1537 $ $DateTime: 2002/10/19 14:18:37 $
 
 package PAR;
-$PAR::VERSION = '0.04';
+$PAR::VERSION = '0.05';
 
 use 5.006;
 use strict;
@@ -13,12 +13,12 @@ PAR - Perl Archive
 
 =head1 VERSION
 
-This document describes version 0.04 of PAR.
+This document describes version 0.05 of PAR.
 
 =head1 SYNOPSIS
 
 (If you want to make an executable that contains all module,
-scrip and data files, please consult L<par.pl> instead.)
+scripts and data files, please consult L<par.pl> instead.)
 
 Following examples assume a F<foo.par> file in Zip format;
 support for compressed gzip (F<*.tgz>) format is planned.
@@ -73,19 +73,32 @@ to load a specific PAR file.
 
 Please see L</SYNOPSIS> for most typical use cases.
 
+=head1 NOTES
+
+For Perl versions without I<PerlIO> support, the B<IO::Scalar> module
+is needed to support C<__DATA__> sections in script and modules
+inside a PAR file.
+
+If you choose to compile F<script/par.pl> with B<perlcc>, it will
+automatically include the correct module (B<PerlIO::scalar> or
+B<IO::Scalar>) if you have it installed.
+
 =cut
 
+my $Reentrant;
 our @PAR_INC;
 our (@LibCache, %LibCache); # I really miss pseudohash.
+our %DATACache; # cache for __DATA__ segments
 
 sub import {
     my $class = shift;
+    return if !@_ and $Reentrant++;
 
     foreach my $par (@_) {
 	push @PAR_INC, $par if unpar($par);
     }
 
-    push @INC, \&incpar unless grep { $_ == \&incpar } @INC;
+    push @INC, \&incpar unless grep { $_ eq \&incpar } @INC;
 
     if (unpar($0)) {
 	push @PAR_INC, $0;
@@ -101,10 +114,21 @@ sub import {
 	    );
 
 	$0 = $file;
-	eval $member->contents;
+	my $program = $member->contents;
+	if ($program =~ s/^__DATA__\n?(.*)//ms) {
+	    $DATACache{$file} = $1;
+	    $program .= _wrap_data($file);
+	}
+
+	{
+	    package main;
+	    eval $program;
+	}
 	die $@ if $@;
 	exit;
     }
+
+    $Reentrant-- if !@_;
 }
 
 sub incpar {
@@ -164,10 +188,43 @@ sub unpar {
     # You did not see this undocumented jenga piece.
     return ($fh, sub {
 	$_ = shift(@lines);
+	if ($_ eq "__DATA__\n" or $_ eq "__END__\n") {
+	    $DATACache{$par} = join('', @lines);
+	    @lines = ();
+	    $_ = _wrap_data($par);
+	}
 	return length $_;
     });
 }
 
+sub _wrap_data {
+    my $key = shift;
+
+    if (eval {require PerlIO::scalar; 1}) {
+	return "use PAR (".
+	       "    open(*DATA, '<', \\\$PAR::DATACache{'$key'}) ? () : ()".
+	       ");\n"; 
+    }
+    elsif (eval {require IO::Scalar; 1}) {
+	return "use IO::Scalar (".
+	       "    tie(*DATA, 'IO::Scalar', \\\$PAR::DATACache{'$key'})".
+	       "    ? () : ()".
+	       ");\n";
+    }
+    else {
+	# only dies when it's used
+	return "use PAR (tie(*DATA, 'PAR::_data') ? () : ())\n";
+    }
+}
+
+package PAR::_data;
+
+sub TIEHANDLE { bless {}, shift }
+sub AUTOLOAD {
+    die "Cannot use __DATA__ sections in .par files; please install IO::Scalar first!\n"
+}
+sub DESTROY {}
+
 1;
 
 =head1 SEE ALSO
